@@ -1,16 +1,11 @@
-import json, os, pprint, shutil
+import json, os, pprint, shutil, yaml
 
 
-class Calculation_Folder:
+class Calculation_Folder(os.PathLike, dict):
     """Represents one folder of a calculation."""
 
     def __init__(self, output_folder, create=False, descriptor_file="descriptor.json"):
-        self.output_folder = os.path.abspath(output_folder)
-        self.descriptor = {}
-
-        self._descriptor_file_name = descriptor_file
-        self._lock_file = "~lock"
-
+        self.output_folder = os.path.normpath(os.path.abspath(output_folder))
         if not os.path.isdir(self.output_folder):
             if not create:
                 raise Exception(
@@ -19,31 +14,50 @@ class Calculation_Folder:
             else:
                 os.makedirs(self.output_folder)
 
-        self.from_json()
+        self._lock_file = "~lock"
+
+        # We enforce that the descriptor file is at the root of the output folder
+        if (
+            not os.path.normpath(os.path.dirname(self.to_abspath(descriptor_file)))
+            == self.output_folder
+        ):
+            raise Exception(
+                "The descriptor file has to be at the root of the calculation folder!"
+            )
+
+        # Deal with the type of the descriptor file
+        self._descriptor_file_name = os.path.normpath(descriptor_file)
+        self._descriptor_file_name_base, self._descriptor_file_ext = os.path.splitext(
+            self._descriptor_file_name
+        )
+
+        if self._descriptor_file_ext == ".json":
+            self.use_json = True
+        elif (
+            self._descriptor_file_ext == ".yml" or self._descriptor_file_ext == ".yaml"
+        ):
+            self.use_json = False
+        else:
+            raise Exception(
+                "Only `.yaml`, `.yml` and `.json` are supported as descriptor file endings"
+            )
+
+        self.from_desc()
 
     def __str__(self):
         return str(self.output_folder)
 
-    def __getitem__(self, key):
-        return self.descriptor[key]
+    def __fspath__(self):
+        return self.output_folder
 
-    def __setitem__(self, key, value):
-        self.descriptor[key] = value
-
-    def __contains__(self, key):
-        return key in self.descriptor
-
-    def items(self):
-        return self.descriptor.items()
-
-    def keys(self):
-        return self.descriptor.keys()
+    def __add__(self, other):
+        return self.output_folder + other
 
     def to_abspath(self, relative_path):
-        return os.path.join(self.output_folder, relative_path)
+        return os.path.join(self, relative_path)
 
     def to_relpath(self, absolute_path):
-        return os.path.relpath(absolute_path, self.output_folder)
+        return os.path.relpath(absolute_path, self)
 
     def copy_file(self, source, rel_dest, create_subdirs=False):
         """Copies a file to a relative path within the calculation folder"""
@@ -61,23 +75,76 @@ class Calculation_Folder:
     def info_string(self):
         res = f"Folder: '{str(self)}'\n"
         res += f"Descriptor: '{self._descriptor_file_name}'\n"
-        res += pprint.pformat(self.descriptor)
+        res += pprint.pformat(self)
         return res
 
+    def from_yaml(self):
+        if os.path.exists(self.to_abspath(self._descriptor_file_name)):
+            with open(self.to_abspath(self._descriptor_file_name), "r") as f:
+                super().__init__(yaml.safe_load(f))
+
+    def from_desc(self):
+        if self.use_json:
+            self.from_json()
+        else:
+            self.from_yaml()
+
+    def to_desc(self):
+        if self.use_json:
+            self.to_json()
+        else:
+            self.to_yaml()
+
+    def to_yaml(self, output_file=None):
+        if output_file is None:
+            output_file = self._descriptor_file_name
+
+        descriptor_file = self.to_abspath(output_file)
+        file, ext = os.path.splitext(output_file)
+
+        # For safety reasons we first write to a temporary file, the main reason is that open(..., "w") will immediately truncate the descriptor file
+        temporary_file = self.to_abspath(file + "__temp__" + ext)
+
+        if ext != ".yml" and ext != ".yaml":
+            raise Exception("File extension has to be `.yml` or `.yaml`")
+
+        try:
+            with open(temporary_file, "w") as f:
+                f.write(yaml.dump(dict(self)))
+                # If json serialization has succeeded, we can remove the old json file and rename the temporary accordingly
+                if os.path.exists(descriptor_file):
+                    os.remove(descriptor_file)
+                os.rename(temporary_file, descriptor_file)
+        except Exception as e:
+            print("JSON serialization has encountered an error.")
+            print(f"The original file '{descriptor_file}' has not been changed.")
+            # We delete the temporary file, if it exists
+            if os.path.exists(temporary_file):
+                os.remove(temporary_file)
+            raise e
+
     def from_json(self):
         if os.path.exists(self.to_abspath(self._descriptor_file_name)):
             with open(self.to_abspath(self._descriptor_file_name), "r") as f:
-                self.descriptor = json.load(f)
+                super().__init__(json.load(f))
 
-    def to_json(self):
+    def to_json(self, output_file=None):
         descriptor_file = self.to_abspath(self._descriptor_file_name)
-        file, ext = os.path.splitext(self._descriptor_file_name)
+        if output_file is None:
+            output_file = self._descriptor_file_name
+
+        descriptor_file = self.to_abspath(output_file)
+        file, ext = os.path.splitext(output_file)
 
         # For safety reasons we first write to a temporary file, the main reason is that open(..., "w") will immediately truncate the descriptor file
         temporary_file = self.to_abspath(file + "__temp__" + ext)
+
+        if ext != ".json":
+            raise Exception("File extension has to be `.json`")
+
         try:
             with open(temporary_file, "w") as f:
-                f.write(json.dumps(self.descriptor, indent=4))
+                f.write(json.dumps(self, indent=4))
                 # If json serialization has succeeded, we can remove the old json file and rename the temporary accordingly
                 if os.path.exists(descriptor_file):
                     os.remove(descriptor_file)
@@ -145,4 +212,4 @@ class Calculation_Folder:
             calc.descriptor = {"number": 123.23, "name" : "bob"}
             calc.format('my_string_{number:.1f}_{name}') = 'my_string_123.2_bob'
         """
-        return self._replace_from_dict(str, self.descriptor)
+        return self._replace_from_dict(str, self)
